<?php

declare(strict_types=1);

namespace App\Controller\Admin\System;

use App\Datatables\DeviceDatatable;
use App\Form\SettingsType;
use App\Repository\DeviceAgentRepository;
use App\Repository\ProductFlagsRepository;
use App\Repository\TypesValueTypeRepository;
use App\Service\Api\ApiService;
use App\Service\Api\Configuration;
use App\Service\Command;
use App\Service\ConfigurationService;
use Psr\Log\LoggerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController as Controller;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Routing\Annotation\Route;

class SettingsController extends Controller {

    private ParameterBagInterface $params;

    private Command $commandService;

    private LoggerInterface $logger;

    private Configuration $apiConfiguration;

    private TypesValueTypeRepository $typesValueTypeRepository;

    /**
     * @param ParameterBagInterface $params
     * @param Command $commandService
     * @param LoggerInterface $logger
     * @param ApiService $apiService
     * @param TypesValueTypeRepository $typesValueTypeRepository
     */
    public function __construct(
        ParameterBagInterface $params,
        Command $commandService,
        LoggerInterface $logger,
        Configuration $apiConfiguration,
        TypesValueTypeRepository $typesValueTypeRepository
    ) {
        $this->params = $params;
        $this->commandService = $commandService;
        $this->logger = $logger;
        $this->apiConfiguration = $apiConfiguration;
        $this->typesValueTypeRepository = $typesValueTypeRepository;
    }

    /**
     * @Route("/admin/settings", name="admin_settings")
     * @param Request $request
     * @param ConfigurationService $configurationService
     * @param Command $commandService
     * @return RedirectResponse|Response
     * @throws \Exception
     */
    public function default(Request $request, ConfigurationService $configurationService, Command $commandService) {

        $smtpServer = $configurationService->getSmtpServer();
        $smtpPort = $configurationService->getSmtpPort();
        $smtpSecurity = $configurationService->getSmtpSecurity();
        $smtpLogin = $configurationService->getSmtpLogin();

        $slackToken = $configurationService->getSlackToken();
        $slackChannel = $configurationService->getSlackChannel();

        $deviceTableColumns = $configurationService->getDeviceTableColumns();
        $deviceTableColumnsChoices = [];
        $availableDeviceTableColumns = DeviceDatatable::availableColumns;
        foreach ($availableDeviceTableColumns as $key => $column) {
            $deviceTableColumnsChoices[$column] = $key;
        }

        // Sort array of choices by actual order stored in db
        $sortingArr = $deviceTableColumns;
        uasort($deviceTableColumnsChoices, function($a, $b) use ($sortingArr) {
            return array_search($a, $sortingArr) <=> array_search($b, $sortingArr);
        });

        $isSlackSettingsComplete = ($slackToken !== null && $slackChannel !== null) ? true : false;
        $isEmailSettingsComplete = ($smtpServer !== null && $smtpPort !== null && $smtpSecurity !== 0 && $smtpLogin !== null) ? true : false;

        $configFormData = $this->getConfigFormData();

        $settings = [
            'serverIpAddress' => $configurationService->getServerIpAddress(),
            'enableEmailNotifications' => $configurationService->isEnabledEmailNotifications(),
            'enableEmailNotificationToAddress' => $configurationService->isEnabledEmailNotificationToAddress(),
            'notificationEmail' => $configurationService->getNotificationEmail(),
            'senderEmail' => $configurationService->getSenderEmail(),
            'smtpServer' => $smtpServer,
            'smtpPort' => $smtpPort,
            'smtpSecurity' => $smtpSecurity,
            'smtpLogin' => $smtpLogin,
            'smtpPassword' => $configurationService->getSmtpPassword(),
            'slackToken' => $slackToken,
            'slackChannel' => $slackChannel,
            'isIpAddressEditEnabled' => $configurationService->isSaasMode(),
            'isSlackSettingsComplete' => $isSlackSettingsComplete,
            'isEmailSettingsComplete' => $isEmailSettingsComplete,
            'deviceTableColumns' => $deviceTableColumns,
            'deviceTableColumnsChoices' => $deviceTableColumnsChoices,
            'configFormData' => $configFormData,
        ];

        $settingsForm = $this->createForm(SettingsType::class, $settings);

        $settingsForm->handleRequest($request);

        if ($settingsForm->isSubmitted() && $settingsForm->isValid()) {
            $inputDataFormNames = array_map(function($ar) {
                return ($ar['writable'] === true) ? $ar['name'] : null;
            }, $configFormData);
            $formData = $settingsForm->getData();

            foreach ($inputDataFormNames as $key => $inputName) {
                if ($inputName === null) {
                    continue;
                }

                $dataApi = $configFormData[$key];
                $isUserDefined = $formData[$inputName . '_enabled'];
                $result = $this->apiConfiguration->setConfiguration($inputName,  ($isUserDefined) ? $formData[$inputName] : null);
                if($result){
                    if ($result->operation_result->Code < 0) {

                        $this->addFlash(
                            'danger',
                            $result->operation_result->Description ,
                        );

                    }
                }else{

                    $this->addFlash(
                        'danger',
                        'Cannot update: ' . $dataApi['label'] ,
                    );

                }

            }

            $sendSlackTestNotification = false;
            $sendEmailTestNotification = false;

            if ($settingsForm->offsetExists('sendSlackTestNotification') && $sendSlackTestNotification = $settingsForm->get('sendSlackTestNotification')->isClicked()) {
                $serverSettings = $settingsForm->getData();
                $configurationService->updateSlackToken($serverSettings['slackToken']);
                $configurationService->updateSlackChannel($serverSettings['slackChannel']);

                $this->sendSlackTestNotification();
            }

            if ($settingsForm->offsetExists('sendEmailTestNotification') && $sendEmailTestNotification = $settingsForm->get('sendEmailTestNotification')->isClicked()) {
                $serverSettings = $settingsForm->getData();
                $configurationService->updateSenderEmail($serverSettings['senderEmail']);
                $configurationService->updateSmtpServer($serverSettings['smtpServer']);
                $configurationService->updateSmtpPort((int)$serverSettings['smtpPort']);
                $configurationService->updateSmtpSecurity($serverSettings['smtpSecurity']);
                $configurationService->updateSmtpLogin($serverSettings['smtpLogin']);
                $configurationService->updateSmtpPassword($serverSettings['smtpPassword']);

                $this->sendEmailkTestNotification();
            }

            if (!$sendSlackTestNotification && !$sendEmailTestNotification) {
                $serverSettings = $settingsForm->getData();

                $configurationService->updateIsEnabledEmailNotifications($serverSettings['enableEmailNotifications']);
                $configurationService->updateIsEnabledEmailNotificationToAddress($serverSettings['enableEmailNotificationToAddress']);
                $configurationService->updateNotificationEmail($serverSettings['notificationEmail']);
                $configurationService->updateSenderEmail($serverSettings['senderEmail']);
                $configurationService->updateSmtpServer($serverSettings['smtpServer']);
                $configurationService->updateSmtpPort((int)$serverSettings['smtpPort']);
                $configurationService->updateSmtpSecurity($serverSettings['smtpSecurity']);
                $configurationService->updateSmtpLogin($serverSettings['smtpLogin']);
                $configurationService->updateSmtpPassword($serverSettings['smtpPassword']);
                $configurationService->updateSlackToken($serverSettings['slackToken']);
                $configurationService->updateSlackChannel($serverSettings['slackChannel']);

                if ($serverSettings['serverIpAddress'] !== $settings['serverIpAddress']) {
                    // Update Server IP Address
                    try {
                        $process = $commandService->setServerIp($serverSettings['serverIpAddress']);
                        $process->mustRun();
                        $exitCode = $process->getExitCode();

                        if ($exitCode === 0) {
                            $this->addFlash(
                                'success',
                                'Settings where saved.',
                            );
                        } else {
                            $this->addFlash(
                                'danger',
                                'Server IP address or FQDN cannot be set, see the event log detail.',
                            );
                        }
                    } catch (ProcessFailedException $exception) {
                        $this->logger->error($exception->getMessage());
                        $this->addFlash(
                            'danger',
                            'Server IP address or FQDN cannot be set, see the event log detail.',
                        );
                    }
                } else {
                    $this->addFlash(
                        'success',
                        'Settings where saved.',
                    );
                }

                return $this->redirectToRoute('admin_settings');
            }
        }

        return $this->render('system/settings.html.twig', [
            'settingsForm' => $settingsForm->createView(),
            //Get group builder which is part of the form named 'config'
            'configItems' => (isset($settingsForm->all()['config'])) ? $settingsForm->all()['config'] : null,
            'nonWritableConfigData' => $this->getNonWritableConfigData($configFormData),
            'deviceTableColumnsChoices' => $deviceTableColumnsChoices,
        ]);
    }

    /**
     * @Route("/admin/device-agents", name="admin_device_agents")
     * @param Request $request
     * @param ConfigurationService $configurationService
     * @param ProductFlagsRepository $productFlagsRepository
     * @param DeviceAgentRepository $deviceAgentRepository
     * @return Response
     * @throws \Exception
     */
    public function deviceAgents(
        Request                $request,
        ConfigurationService   $configurationService,
        ProductFlagsRepository $productFlagsRepository,
        DeviceAgentRepository  $deviceAgentRepository
    ) {
        $agents = $deviceAgentRepository->findAll();
        $serverIpAddress = $configurationService->getServerIpAddress();
        $processServerIpUpdate = $productFlagsRepository->getProcessServerIpUpdate();

        return $this->render('system/deviceAgents.html.twig', [
            'agents' => $agents,
            'serverIpAddress' => $serverIpAddress,
            'processServerIpUpdate' => false, // $processServerIpUpdate->version,
        ]);
    }

    /**
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|void
     */
    private function sendSlackTestNotification() {
        try {
            $process = $this->commandService->sendSlackNotification('Testing application notification');
            $process->mustRun();
            $output = $process->getOutput();

            $this->addFlash(
                'success',
                'Notification to Slack was sent.',
            );
        } catch (ProcessFailedException $exception) {
            $this->logger->error($exception->getMessage());
            $this->addFlash(
                'danger',
                'Cannot send notification to Slack. ' . $exception->getProcess()->getErrorOutput(),
            );
        }

        return $this->redirectToRoute('admin_settings');
    }

    /**
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|void
     */
    private function sendEmailkTestNotification() {
        try {
            $process = $this->commandService->sendEmailNotification('Testing application notification');
            $process->setTimeout(10);
            $process->mustRun();
            $output = $process->getOutput();

            $this->addFlash(
                'success',
                'Notification to email was sent.',
            );
        } catch (ProcessFailedException $exception) {
            $this->logger->error($exception->getMessage());
            $this->addFlash(
                'danger',
                'Cannot send notification to email. ' . $exception->getProcess()->getErrorOutput(),
            );
        } catch (\Exception $exception) {
            $this->logger->error($exception->getMessage());
            $this->addFlash(
                'danger',
                'Cannot send notification to email. ' . $exception->getProcess()->getErrorOutput(),
            );
        }

        return $this->redirectToRoute('admin_settings');
    }

    /**
     * @return array
     */
    private function getConfigFormData(): array {

        $config = $this->apiConfiguration->getConfiguration();
        $types = $this->typesValueTypeRepository->getAllTypes();

        // Cannot get data from API
        if (!$config) {
            return [];
        }

        $configFormData = [];

        foreach ($config->configuration as $configItem) {
            // dump($configItem);
            $configFormData[] = [
                'name' => $configItem->Option,
                'label' => $configItem->Caption,
                'description' => $configItem->Description,
                'type' => $types[$configItem->Type]['name'],
                'value' => $configItem->{'User Value'},
                'actualValue' => $configItem->{'Actual Value'},
                'unit' => $configItem->Unit,
                'writable' => (bool)$configItem->Writable,
                'default' => $configItem->Default,
                'min' => $configItem->Minimum,
                'max' => $configItem->Maximum,
            ];
        }

        return $configFormData;
    }

    /**
     * @param array $configFormData
     * @return array
     */
    private function getNonWritableConfigData(array $configFormData): array {

        return array_filter($configFormData, function($v) {
            return $v['writable'] === false;
        });

    }

}
